@@ -6,7 +6,7 @@ import json
 from contextlib import contextmanager
 from dataclasses import asdict, is_dataclass
 from pathlib import Path
-from tempfile import NamedTemporaryFile, TemporaryDirectory
+from tempfile import TemporaryDirectory
 from typing import Any, Iterator, Optional, Union
 from urllib.parse import urlparse
 
@@ -14,12 +14,18 @@ import dbt.flags as flags
 from dbt.contracts.results import RunExecutionResult, RunResult, agate
 from dbt.logger import log_manager
 from dbt.main import initialize_config_values, parse_args, track_run
+from dbt.semver import VersionSpecifier
+from dbt.version import installed as installed_version
 
 from airflow import AirflowException
 from airflow.models.baseoperator import BaseOperator
 from airflow.models.xcom import XCOM_RETURN_KEY
 from airflow.utils.decorators import apply_defaults
 
+IS_DBT_VERSION_LESS_THAN_0_21 = (
+    int(installed_version.minor) < 21 and int(installed_version.major) == 0
+)
+
 
 class DbtBaseOperator(BaseOperator):
     """The basic Airflow dbt operator.
@@ -89,7 +95,14 @@ class DbtBaseOperator(BaseOperator):
         self._dbt_s3_hook = None
 
     def execute(self, context: dict):
-        """Execute dbt command with prepared arguments."""
+        """Execute dbt command with prepared arguments.
+
+        Execution requires setting up a directory with the dbt project files and
+        overriding the logging.
+
+        Args:
+            context: The Airflow's task context
+        """
         with self.dbt_directory() as dbt_dir:  # type: str
             with self.override_dbt_logging(dbt_dir):
                 args = self.prepare_args()
@@ -119,6 +132,9 @@ class DbtBaseOperator(BaseOperator):
     def xcom_push_artifacts(self, context: dict, dbt_directory: str):
         """Read dbt artifacts and push them to XCom.
 
+        Artifacts are read from the target/ directory in dbt_directory. This method will
+        fail if the required artifact is not found.
+
         Args:
             context: The Airflow task's context.
             dbt_directory: A directory containing a dbt project. Artifacts will be
@@ -154,7 +170,11 @@ class DbtBaseOperator(BaseOperator):
         return args
 
     def args_list(self) -> list[str]:
-        """Build a list of arguments to pass to dbt."""
+        """Build a list of arguments to pass to dbt.
+
+        Building involves creating a list of flags for dbt to parse given the operators
+        attributes and the values specified by __dbt_args__.
+        """
         args = []
         for arg in self.__dbt_args__:
             value = getattr(self, arg, None)
@@ -235,23 +255,18 @@ class DbtBaseOperator(BaseOperator):
     def override_dbt_logging(self, dbt_directory: str = None):
         """Override dbt's logger.
 
-        dbt logger writes to STDOUT and I haven't found a way
-        to bubble up to the Airflow command logger. As a workaround,
-        I set the output stream to a temporary file that is later
-        read and logged using the command's logger.
+        We override the output stream of the dbt logger to use Airflow's StreamLogWriter
+        so that we can ensure dbt logs properly to the Airflow task's log output.
         """
-        with NamedTemporaryFile(dir=dbt_directory, mode="w+") as f:
-            with log_manager.applicationbound():
+        from airflow.utils.log.logging_mixin import StreamLogWriter
 
-                log_manager.reset_handlers()
-                log_manager.set_path(dbt_directory)
-                log_manager.set_output_stream(f)
-
-                yield
-
-            with open(f.name) as read_file:
-                for line in read_file:
-                    self.log.info(line.rstrip())
+        with log_manager.applicationbound():
+            log_manager.reset_handlers()
+            log_manager.set_path(dbt_directory)
+            log_manager.set_output_stream(
+                StreamLogWriter(self.log, self.log.getEffectiveLevel())
+            )
+            yield
 
     def run_dbt_command(self, args: list[Optional[str]]) -> tuple[RunResult, bool]:
         """Run a dbt command as implemented by a subclass."""
@@ -288,13 +303,19 @@ class DbtBaseOperator(BaseOperator):
 
 
 class DbtRunOperator(DbtBaseOperator):
-    """Executes dbt run."""
+    """Executes a dbt run command.
+
+    The run command executes SQL model files against the given target. The
+    documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/run.
+    """
 
     command = "run"
 
     __dbt_args__ = DbtBaseOperator.__dbt_args__ + [
         "full_refresh",
         "models",
+        "select",
         "fail_fast",
         "threads",
         "exclude",
@@ -308,6 +329,7 @@ class DbtRunOperator(DbtBaseOperator):
         self,
         full_refresh: Optional[bool] = None,
         models: Optional[list[str]] = None,
+        select: Optional[list[str]] = None,
         fail_fast: Optional[bool] = None,
         threads: Optional[int] = None,
         exclude: Optional[list[str]] = None,
@@ -319,7 +341,6 @@ class DbtRunOperator(DbtBaseOperator):
     ) -> None:
         super().__init__(**kwargs)
         self.full_refresh = full_refresh
-        self.models = models
         self.fail_fast = fail_fast
         self.threads = threads
         self.exclude = exclude
@@ -328,9 +349,19 @@ class DbtRunOperator(DbtBaseOperator):
         self.defer = defer
         self.no_defer = no_defer
 
+        if IS_DBT_VERSION_LESS_THAN_0_21:
+            self.models = models or select
+        else:
+            self.select = select or models
+
 
 class DbtSeedOperator(DbtBaseOperator):
-    """Executes dbt seed."""
+    """Executes a dbt seed command.
+
+    The seed command loads csv files into the  the given target. The
+    documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/seed.
+    """
 
     command = "seed"
 
@@ -366,7 +397,12 @@ class DbtSeedOperator(DbtBaseOperator):
 
 
 class DbtTestOperator(DbtBaseOperator):
-    """Executes dbt test."""
+    """Executes a dbt test command.
+
+    The test command runs data and/or schema tests. The
+    documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/test.
+    """
 
     command = "test"
 
@@ -375,6 +411,7 @@ class DbtTestOperator(DbtBaseOperator):
         "schema",
         "fail_fast",
         "models",
+        "select",
         "threads",
         "exclude",
         "selector",
@@ -388,6 +425,7 @@ class DbtTestOperator(DbtBaseOperator):
         data: Optional[bool] = None,
         schema: Optional[bool] = None,
         models: Optional[list[str]] = None,
+        select: Optional[list[str]] = None,
         fail_fast: Optional[bool] = None,
         threads: Optional[int] = None,
         exclude: Optional[list[str]] = None,
@@ -400,7 +438,6 @@ class DbtTestOperator(DbtBaseOperator):
         super().__init__(**kwargs)
         self.data = data
         self.schema = schema
-        self.models = models
         self.fail_fast = fail_fast
         self.threads = threads
         self.exclude = exclude
@@ -409,9 +446,19 @@ class DbtTestOperator(DbtBaseOperator):
         self.defer = defer
         self.no_defer = no_defer
 
+        if IS_DBT_VERSION_LESS_THAN_0_21:
+            self.models = models or select
+        else:
+            self.select = select or models
+
 
 class DbtCompileOperator(DbtBaseOperator):
-    """Executes dbt compile."""
+    """Executes a dbt compile command.
+
+    The compile command generates SQL files. The
+    documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/compile.
+    """
 
     command = "compile"
 
@@ -421,6 +468,7 @@ class DbtCompileOperator(DbtBaseOperator):
         "fail_fast",
         "threads",
         "models",
+        "select",
         "exclude",
         "selector",
         "state",
@@ -431,6 +479,7 @@ class DbtCompileOperator(DbtBaseOperator):
         parse_only: Optional[bool] = None,
         full_refresh: Optional[bool] = None,
         models: Optional[list[str]] = None,
+        select: Optional[list[str]] = None,
         fail_fast: Optional[bool] = None,
         threads: Optional[int] = None,
         exclude: Optional[list[str]] = None,
@@ -441,16 +490,24 @@ class DbtCompileOperator(DbtBaseOperator):
         super().__init__(**kwargs)
         self.parse_only = parse_only
         self.full_refresh = full_refresh
-        self.models = models
         self.fail_fast = fail_fast
         self.threads = threads
         self.exclude = exclude
         self.selector = selector
         self.state = state
 
+        if IS_DBT_VERSION_LESS_THAN_0_21:
+            self.models = models or select
+        else:
+            self.select = select or models
+
 
 class DbtDepsOperator(DbtBaseOperator):
-    """Executes dbt deps."""
+    """Executes a dbt deps command.
+
+    The documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/deps.
+    """
 
     command = "deps"
 
@@ -459,7 +516,11 @@ class DbtDepsOperator(DbtBaseOperator):
 
 
 class DbtCleanOperator(DbtBaseOperator):
-    """Executes dbt clean."""
+    """Executes a dbt clean command.
+
+    The documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/debug.
+    """
 
     command = "clean"
 
@@ -468,7 +529,11 @@ class DbtCleanOperator(DbtBaseOperator):
 
 
 class DbtDebugOperator(DbtBaseOperator):
-    """Execute dbt debug."""
+    """Executes a dbt debug command.
+
+    The documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/debug.
+    """
 
     command = "debug"
 
@@ -486,7 +551,11 @@ class DbtDebugOperator(DbtBaseOperator):
 
 
 class DbtSnapshotOperator(DbtBaseOperator):
-    """Execute dbt snapshot."""
+    """Executes a dbt snapshot command.
+
+    The documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/snapshot.
+    """
 
     command = "snapshot"
 
@@ -516,36 +585,40 @@ class DbtSnapshotOperator(DbtBaseOperator):
 
 
 class DbtLsOperator(DbtBaseOperator):
-    """Execute dbt list (or ls)."""
+    """Executes a dbt list (or ls) command.
+
+    The documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/list.
+    """
 
     command = "ls"
 
     __dbt_args__ = DbtBaseOperator.__dbt_args__ + [
         "resource_type",
         "select",
-        "models",
         "exclude",
         "selector",
         "dbt_output",
+        "output_keys",
     ]
 
     def __init__(
         self,
         resource_type: Optional[list[str]] = None,
         select: Optional[list[str]] = None,
-        models: Optional[list[str]] = None,
         exclude: Optional[list[str]] = None,
         selector: Optional[str] = None,
         dbt_output: Optional[str] = None,
+        output_keys: Optional[list[str]] = None,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
         self.resource_type = resource_type
         self.select = select
-        self.models = models
         self.exclude = exclude
         self.selector = selector
         self.dbt_output = dbt_output
+        self.output_keys = output_keys
 
 
 # Convinience alias
@@ -553,7 +626,11 @@ DbtListOperator = DbtLsOperator
 
 
 class DbtRunOperationOperator(DbtBaseOperator):
-    """Execute dbt run-operation."""
+    """Executes a dbt run-operation command.
+
+    The documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/run-operation.
+    """
 
     command = "run-operation"
 
@@ -572,7 +649,11 @@ class DbtRunOperationOperator(DbtBaseOperator):
 
 
 class DbtParseOperator(DbtBaseOperator):
-    """Execute dbt parse."""
+    """Executes a dbt parse command.
+
+    The documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/parse.
+    """
 
     command = "parse"
 
@@ -584,22 +665,29 @@ class DbtParseOperator(DbtBaseOperator):
 
 
 class DbtSourceOperator(DbtBaseOperator):
-    """Execute dbt source."""
+    """Executes a dbt source command.
+
+    The documentation for the dbt command can be found here:
+    https://docs.getdbt.com/reference/commands/source.
+    """
 
     command = "source"
 
     __dbt_args__ = DbtBaseOperator.__dbt_args__ + [
         "select",
-        "models",
+        "threads",
         "exclude",
         "selector",
+        "state",
         "dbt_output",
     ]
 
     def __init__(
         self,
         # Only one subcommand is currently provided
-        subcommand: str = "snapshot-freshness",
+        subcommand: str = "freshness"
+        if not installed_version < VersionSpecifier.from_version_string("0.21.0")
+        else "snapshot-freshness",
         select: Optional[list[str]] = None,
         dbt_output: Optional[Union[str, Path]] = None,
         threads: Optional[int] = None,
@@ -610,11 +698,67 @@ class DbtSourceOperator(DbtBaseOperator):
     ) -> None:
         super().__init__(positional_args=[subcommand], **kwargs)
         self.select = select
-        self.dbt_output = dbt_output
         self.threads = threads
         self.exclude = exclude
         self.selector = selector
         self.state = state
+        self.dbt_output = dbt_output
+
+
+class DbtBuildOperator(DbtBaseOperator):
+    """Executes a dbt build command.
+
+    The build command combines the run, test, seed, and snapshot commands into one. The
+    full Documentation for the dbt build command can be found here:
+    https://docs.getdbt.com/reference/commands/build.
+    """
+
+    command = "build"
+
+    __dbt_args__ = DbtBaseOperator.__dbt_args__ + [
+        "full_refresh",
+        "select",
+        "fail_fast",
+        "threads",
+        "exclude",
+        "selector",
+        "state",
+        "defer",
+        "no_defer",
+        "data",
+        "schema",
+        "show",
+    ]
+
+    def __init__(
+        self,
+        full_refresh: Optional[bool] = None,
+        select: Optional[list[str]] = None,
+        fail_fast: Optional[bool] = None,
+        threads: Optional[int] = None,
+        exclude: Optional[list[str]] = None,
+        selector: Optional[str] = None,
+        state: Optional[Union[str, Path]] = None,
+        defer: Optional[bool] = None,
+        no_defer: Optional[bool] = None,
+        data: Optional[bool] = None,
+        schema: Optional[bool] = None,
+        show: Optional[bool] = None,
+        **kwargs,
+    ) -> None:
+        super().__init__(**kwargs)
+        self.full_refresh = full_refresh
+        self.select = select
+        self.fail_fast = fail_fast
+        self.threads = threads
+        self.exclude = exclude
+        self.selector = selector
+        self.state = state
+        self.defer = defer
+        self.no_defer = no_defer
+        self.data = data
+        self.schema = schema
+        self.show = show
 
 
 def run_result_factory(data: list[tuple[Any, Any]]):
